﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media.Imaging;
using System.Windows.Threading;

namespace Memoriajáték
{
	public partial class wndFőablak : Window
	{
		/// <summary>
		/// A hátlapként alkalmazott kép állomány neve.
		/// </summary>
		private string HátlapNév="hatter.jpg";

		/// <summary>
		/// A hátlapkép objektum referenciáját tároló adattag.
		/// </summary>
		private BitmapImage biHátlapKép;

		/// <summary>
		/// A játékban felhasznált mesefigurákat tartalmazó 
		/// kép állományok nevei.
		/// </summary>
		private string[] KépNevek =
		{
			"grabowski.jpg", "kukori.jpg",
			"kuldonc.jpg", "vili.jpg"
		};

		/// <summary>
		/// Tömb a nyolc kép objektum referenciáinak tárolására.
		/// </summary>
		BitmapImage[]biKépek=new BitmapImage[8];

		/// <summary>
		/// Tömb a mesefigurák megjelenítésére használt Image
		/// komponensek referenciáinak tárolására.
		/// </summary>
		private Image[] imKéphelyek;

		/// <summary>
		/// Véletlenszámok előállítására szolgáló objektum.
		/// </summary>
		Random Véletlen =new Random();

		/// <summary>
		/// Időzítő a képek memorizálására adott idő követéséhez. 
		/// </summary>
		private DispatcherTimer dtIdőzítő;

		/// <summary>
		/// A főablak konstruktora. Létrehozza a képeket megjelenítő 
		/// komponensek tömbjét, és betölti a képeket a memóriába.
		/// </summary>
		public wndFőablak()
		{
			// Komponensek létrehozása és inicializálása.
			InitializeComponent();
			// Képeket megjelenítő komponensek tömbjének létrehozása.
			imKéphelyek = new Image[]
			{
				im10,im11,im12,im13,
				im20,im21,im22,im23
			};
			// Időzítő objektum létrehozása, az időzítés beállítása 3 másodpercre
			dtIdőzítő = new DispatcherTimer
			{
				Interval = new TimeSpan(0, 0, 0, 0, 3000),
				IsEnabled = false
			};
			dtIdőzítő.Tick += dtIdőzítő_Tick;
			// Képek betöltése a memóriába.
			KépeketBetölt();
			// Képek láthatóvá tétele.
			KépeketMegmutat();
			// Minden komponens a hátlapképet mutatja, ha letelt a beállított időintervallum.
			dtIdőzítő.Start();
		}

		/// <summary>
		/// Az időzítés leteltével a hátlapképet helyezi el mind a nyolc kép komponensre,
		/// majd leállítja az időzítőt.
		/// </summary>
		private void dtIdőzítő_Tick(object sender, EventArgs e)
		{
			HátlapképpelKitölt();
			dtIdőzítő.Stop();
		}

		/// <summary>
		/// Mind a nyolc helyen a hátlapképet jeleníti meg.
		/// </summary>
		private void HátlapképpelKitölt()
		{
			for (int i = 0; i < 8; i++)
				imKéphelyek[i].Source = biHátlapKép;
		}

		/// <summary>
		/// Láthatóvá teszi a nyolc képet.
		/// </summary>
		private void KépeketMegmutat()
		{
			for (int i = 0; i < 8; i++)
			{
				imKéphelyek[i].Source = biKépek[i];
			}
		}
		/// <summary>
		/// Betölti memóriába a játékhoz használt képeket. A képek a 
		/// projekt gyökérkönyvtárában levő Kepek alkönyvtárában kell legyenek.
		/// </summary>    
		private void KépeketBetölt()
		{
			try
			{
				// Hátlapkép betöltése.
				biHátlapKép = new BitmapImage(
						new Uri(@"Kepek/" + HátlapNév, UriKind.Relative));
				// A négy mesefigura kép betöltése.
				for (int i = 0; i < 4; i++)
				{ biKépek[i] = new BitmapImage(
						new Uri(@"Kepek/" + KépNevek[i], UriKind.Relative));
					// A másodpéldányokat beazonosító referenciák.
					biKépek[i + 4] = biKépek[i];
				}
			}
			catch (Exception)
			{ MessageBox.Show("A képek nem találhatóak a megadott útvonalon!", "Hiba",
					MessageBoxButton.OK, MessageBoxImage.Error);
			}
		}

		/// <summary>
		/// Kilép az alkalmazásból.
		/// </summary>
		/// <param name="sender">A Kilép menüpont objektum.</param>
		/// <param name="e"></param>
		private void miKilép_Click(object sender, RoutedEventArgs e)
		{
			Application.Current.Shutdown();
		}

		/// <summary>
		/// A Kever menüpont hatására az induláskor automatikusan 
		/// betöltött négy pár képet véletlenszerű elrendezésben 
		/// megjeleníti. Várakozik, majd a képek helyett zöldre 
		/// festett téglalapokat jelenít meg (hatter.jpg). 
		/// </summary>
		/// <param name="sender">A menüpont objektum.</param>
		/// <param name="e"></param>
		private void miKever_Click(object sender, RoutedEventArgs e)
		{
			// Véletlen sorrend meghatározása.
			VéletlenSorrendbeRak();
			// Képek láthatóvá tétele.
			KépeketMegmutat();
			// Várakozás az időzítőben beállított ideig, majd a hátlapképek megjelenítése.
			dtIdőzítő.Start();
		}

		/// <summary>
		/// Meghatározza a képek véletlenszerű sorrendjét.
		/// </summary>
		private void VéletlenSorrendbeRak()
		{
			// Létrehozunk egy lista objektumot a képek referenciáinak tárolássára.
			List<BitmapImage> KépLista = new List<BitmapImage>();
			// Mind a 8 kép referenciáját elhelyezzük a listában.
			KépLista.AddRange(biKépek);
			// "Húzás a kalapból" véletlenszerűen kivesszük a nyolc referenciát.
			for (int i = 0; i < 8; i++)
			{
				// A maradék listából véletleszerűen kiválasztunk egy elemet.
				int Sorszám = Véletlen.Next(0, KépLista.Count);
				// Betesszük a tömb i. helyére
				biKépek[i] = KépLista[Sorszám];
				// Eltávolítjuk a listáról.
				KépLista.RemoveAt(Sorszám);
			}
		}


		/// <summary>
		/// A Kérdez menüpont hatására egy új ablakot jelenít meg 
		/// Kérdés fejléccel. Az ablakban az előzőekben látható 
		/// négy képtípus valamelyike jelenik meg véletlenszerűen. 
		/// A felhasználó kiválasztja, hogy szerinte mely helyeken fordult elő a 
		/// főablakban a kép, majd kattint az OK gombon.
		/// Ekkor eltűnik a Kérdez ablak, és a válasz helyességétől 
		/// függően az alábbi két üzenetablak egyike jelenik meg, 
		/// majd újból láthatóvá válik a nyolc kép úgy, ahogy azt a 
		/// legelső ábrán láthatjuk.
		/// </summary>
		/// <param name="sender">A Kérdez menüpont objektum.</param>
		/// <param name="e"></param>
		private void miKérdez_Click(object sender, RoutedEventArgs e)
		{
			// Ablak objektum létrehozása.
			var wndKérdés = new wndKérdés();
			// Kép sorszám kiválasztása véletlenszerűen.
			int Sorszám = Véletlen.Next(0, 8);
			// Keresett kép kiválasztása.
			BitmapImage KeresettKép = biKépek[Sorszám];
			// Kép komponenshez rendelése.
			wndKérdés.biKép = KeresettKép;
			// Megjeleníti a párbeszédablakot.
			if (wndKérdés.ShowDialog()==true)
			{
				// Lekérdezi, hogy mely pozíciókat választotta ki a 
				// felhasználó, majd előállítja az adott pozícióban található
				// képek referenciáit.
				BitmapImage biElső = biKépek[wndKérdés.ElsőSorszám];
				BitmapImage biMásodik = biKépek[wndKérdés.MásodikSorszám];
				// Megvizsgálja, hogy a keresett kép azonos-e a két megjelölt 
				// képpel.
				if (biElső == KeresettKép && biMásodik == KeresettKép)
					MessageBox.Show("Hurrá eltaláltad!");
				else
					MessageBox.Show("Hát ez most nem jött össze!");
				// Újból láthatóvá teszi a képeket.
				KépeketMegmutat();
			}
		}
	}
}
